#
#       (c) Copyright 2008, Four Js AsiaPac - www.4js.com.au/local
#
#       MIT License (http://www.opensource.org/licenses/mit-license.php)
#
#       Permission is hereby granted, free of charge, to any person
#       obtaining a copy of this software and associated documentation
#       files (the "Software"), to deal in the Software without restriction,
#       including without limitation the rights to use, copy, modify, merge,
#       publish, distribute, sublicense, and/or sell copies of the Software,
#       and to permit persons to whom the Software is furnished to do so,
#       subject to the following conditions:
#
#       The above copyright notice and this permission notice shall be
#       included in all copies or substantial portions of the Software.
#
#       THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
#       EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
#       OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
#       NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
#       BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
#       ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
#       CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
#       THE SOFTWARE.
#
#       makefile Makefile for example Google Maps Genero program
#
#       April 2008 reuben@4js.com.au
#
# Example of a Genero program utilising the Genero MAPS APIs for static maps
# More details can be found here http://code.google.com/apis/maps/


IMPORT com
IMPORT xml

-- A unique key that is available from http://code.google.com/apis/maps/signup.html
-- Please replace with your own key as usage of a certain key is limited
CONSTANT c_googlekey = "ABQIAAAAgg35UlaF7bP0AXZjRMf8WRSSAVt8jDhBFa1zz53EcnyyKZXBCBRyGAdLwDZ67Xwbpnm0bCOytzag0A"


-- A record holding address information
DEFINE m_address RECORD
   address1 STRING,
   address2 STRING,
   city  STRING,
   state STRING,
   country STRING,
   latlong STRING,
   postcode STRING
END RECORD

DEFINE m_zoom SMALLINT -- Zoom parameter, note this value is negated so
                       -- that the slider can operate zoom in up, zoom out down

MAIN
DEFINE url STRING
 
   CLOSE WINDOW SCREEN 
   DEFER INTERRUPT 
   DEFER QUIT 
   OPTIONS INPUT WRAP 
   OPTIONS FIELD ORDER FORM

   CALL ui.Interface.LoadStyles("googlemaps")
   OPEN WINDOW w WITH FORM "googlemaps"

   -- Initial zoom setting
   LET m_zoom = -14

   INPUT m_address.address1, m_address.address2, m_address.city, m_address.state, m_address.country, m_address.postcode,  m_address.latlong , m_zoom
   FROM address1, address2, city, state, country, postcode, latlong, zoom ATTRIBUTES(UNBUFFERED, WITHOUT DEFAULTS=TRUE, ACCEPT=FALSE, CANCEL=FALSE)
      BEFORE INPUT
	     MESSAGE "Enter an address, get the latitude/longitude, and then draw map"
	     CALL dialog_state(DIALOG)

      -- Get the lat/long for address
      ON ACTION getdetails
	     IF NOT geocode() THEN
		    INITIALIZE m_address.latlong TO NULL
		 END IF
	     CALL dialog_state(DIALOG)

      -- Select a 4Js office address
	  ON ACTION preload
	     CALL preload()
	     CALL dialog_state(DIALOG)

      -- Draw the map
	  ON ACTION drawmap
	     CALL drawmap()
	     CALL dialog_state(DIALOG)

      -- Respond to changes with the slider
	  ON CHANGE zoom
	     CALL drawmap()
	     CALL dialog_state(DIALOG)

	  ON ACTION close
	     EXIT INPUT
   END INPUT
END MAIN


-- Set properties of dialog
FUNCTION dialog_state(d)
DEFINE d ui.Dialog

   -- If lat/long popualted then allow drawmap, zoom
   IF m_address.latlong.getLength() > 0 THEN
       CALL d.setActionActive("drawmap", TRUE)
	   CALL d.setFieldActive("zoom", TRUE)
   ELSE
       CALL d.setActionActive("drawmap", FALSE)
	   CALL d.setFieldActive("zoom", FALSE)
   END IF
   CALL d.setFieldActive("latlong", FALSE)
END FUNCTION


-- Get the lat/long of an address using google maps geocoding API http://code.google.com/apis/maps/documentation/services.html#Geocoding
FUNCTION geocode()
DEFINE l_http_req com.HTTPRequest
DEFINE l_http_resp com.HTTPResponse

DEFINE l_result_dom xml.DomDocument
DEFINE l_root, l_result_node, l_result_element_node, l_text_node xml.DomNode
DEFINE l_result_list xml.DomNodeList
DEFINE url STRING
DEFINE l_tok base.StringTokenizer

DEFINE l_accuracy SMALLINT

   LET url = SFMT("http://maps.google.com/maps/geo?q=%1,%2,%3,%4,%5,%6&output=xml&key=%7",m_address.address1, m_address.address2, m_address.city, m_address.state, m_address.country,m_address.postcode,c_googlekey)

   TRY 
      LET l_http_req = com.HTTPRequest.Create(url)
      CALL l_http_req.doRequest()
      LET l_http_resp = l_http_req.getResponse()
      IF l_http_resp.getStatusCode() != 200 THEN
		 DISPLAY SFMT("HTTP ERROR(%1) %2",l_http_resp.getStatusCode(), l_http_resp.getStatusDescription())
		 RETURN FALSE
      ELSE
	     LET l_result_dom= l_http_resp.getXmlResponse()
      END IF
   CATCH
      DISPLAY SFMT("ERROR (%1) %2",l_http_resp.getStatusCode(), l_http_resp.getStatusDescription())
      RETURN FALSE
   END TRY
 
   LET l_root = l_result_dom.getDocumentElement()

   -- Display the result XML
   -- Uncomment the following line to debug and interpret the XML returned
   ---DISPLAY l_root.toString()

   -- Find the co-ordinates from what was returned
   LET l_result_list = l_root.getElementsByTagNameNS("coordinates","http://earth.google.com/kml/2.0")
   IF l_result_list.getCount() > 0 THEN
      LET l_result_element_node = l_result_list.getItem(1)
	  LET l_text_node = l_result_element_node.getChildNodeItem(1)
	  # swap lat/long around, wonder why Google do it like this? seems odd
	  LET l_tok = base.StringTokenizer.create(l_text_node.getNodeValue(),",")
	  LET m_address.latlong = l_tok.nextToken()
	  LET m_address.latlong = l_tok.nextToken(),",",m_address.latlong

	  -- At this point we will return TRUE but display a warning message
	  -- if the address was not resolved exactly, as indicated by an accuracy 
	  -- value less than 8, see http://code.google.com/apis/maps/documentation/reference.html for details
       LET l_result_list = l_root.getElementsByTagNameNS("AddressDetails","urn:oasis:names:tc:ciq:xsdschema:xAL:2.0")
      IF l_result_list.getCount() > 0 THEN
         LET l_result_element_node = l_result_list.getItem(1)
		 LET l_accuracy = l_result_element_node.getAttribute("Accuracy")
		 IF l_accuracy = 8 THEN
		    #OK
	     ELSE
		    ERROR "WARNING: The latitude/longitude was not able to be resolved to the exact street address"
		 END IF
	  END IF
	  RETURN TRUE
   END IF
   ERROR "ERROR Address not found"
   RETURN FALSE
END FUNCTION 



-- Draw the map using Google Static Maps API as documented http://code.google.com/apis/maps/documentation/staticmaps/
FUNCTION drawmap()
DEFINE url STRING

   LET url = SFMT("http://maps.google.com/staticmap?center=%1&zoom=%3&size=512x512&key=%2&markers=%1,reda",m_address.latlong, c_googlekey, -m_zoom USING "<<")
   DISPLAY url TO url
END FUNCTION


-- Populate the address fields with a selected 4Js office
FUNCTION preload()
DEFINE l_office STRING

   MENU "Select 4Js Office" ATTRIBUTES(STYLE="popup")
	  COMMAND "Strasbourg - France" LET l_office = "strasbourg"
	  COMMAND "Irving (TX) - USA" LET l_office = "irving"
      COMMAND "Sydney - Australia" LET l_office = "sydney"
      COMMAND "Kuala Lumpur - Malaysia" LET l_office = "kl"
      COMMAND "Auckland - New Zealand" LET l_office = "auckland"
	  COMMAND "Other" LET l_office = "other"
      ON ACTION cancel
	     # do nothing
   END MENU
   IF l_office.getLength() = 0 THEN
      RETURN
   END IF
   IF l_office = "other" THEN
      MENU "Genero Google Maps" ATTRIBUTES(COMMENT="Send address details to reuben@4js.com.au for inclusion.  Please check address at google maps first", STYLE="dialog")
	     ON ACTION accept
		    EXIT MENU
	     ON ACTION close
		    EXIT MENU
	  END MENU
	  RETURN 
   END IF
   INITIALIZE m_address.* TO NULL
   CASE l_office
      WHEN "strasbourg"
		 LET m_address.address1 = "1 Rue de Berne"
		 LET m_address.address2 = "Schitigheim"
		 LET m_address.city = "Strasbourg"
		 LET m_address.state = ""
		 LET m_address.country = "France"
		 LET m_address.postcode = "67300"
      WHEN "irving"
		 LET m_address.address1 = "251 O Connor Ridge Boulevard"
		 LET m_address.address2 = ""
		 LET m_address.city = "Irving"
		 LET m_address.state = "Texas"
		 LET m_address.country = "USA"
		 LET m_address.postcode = "75038"
      WHEN "sydney"
		 LET m_address.address1 = "165 Walker Street"
		 LET m_address.address2 = "North Sydney"
		 LET m_address.city = "Sydney"
		 LET m_address.state = "New South Wales"
		 LET m_address.country = "Australia"
      WHEN "kl"
		 LET m_address.address2 = "Bukit Damansara"
      	 LET m_address.city = "Kuala Lumpur"
		 LET m_address.country = "Malaysia"
      WHEN "auckland"
		 LET m_address.address1 = "369 Queen Street"
		 LET m_address.city = "Auckland"
		 LET m_address.country = "New Zealand"
   END CASE
END FUNCTION
